#include <kernel/proc.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
//----------------------------------------------------------------------
void
print_proc_head (void)
{
    printf (
"pp  p pg                                                            \n"
"id id rp  tty  uid euid suid usage s iaddr isiz daddr dsiz sp   name\n"
           );
}
//----------------------------------------------------------------------
void
print_proc_pid (proc_t *ps, pid_t pid)
{
    char  stat;
    switch (ps->status)
      {
        case PROC_EMPTY   : stat = '-'; break;
        case PROC_CREATED : stat = 'c'; break;
        case PROC_READY   : stat = 'r'; break;
        case PROC_RUNNING : stat = 'R'; break;
        case PROC_SLEEPING: stat = 's'; break;
        case PROC_ZOMBIE  : stat = 'z'; break;
        default           : stat = '?'; break;
      }

    printf ("%2i %2i %2i %04x %4i %4i %4i %02i.%02i %c %05lx %04x ",
              (unsigned int) ps->ppid,
              (unsigned int) pid,
              (unsigned int) ps->pgrp,
              (unsigned int) ps->device_tty,
              (unsigned int) ps->uid,
              (unsigned int) ps->euid,
              (unsigned int) ps->suid,
              (unsigned int) ((ps->usage / CLOCKS_PER_SEC) / 60),
              (unsigned int) ((ps->usage / CLOCKS_PER_SEC) % 60),
              stat,
              (unsigned long int) ps->address_i,
              (unsigned int) ps->size_i);

    printf ("%05lx %04x %04x %s",
              (unsigned long int) ps->address_d,
              (unsigned int) ps->size_d,
              (unsigned int) ps->sp,
                             ps->name);

    printf ("\n");
}
//----------------------------------------------------------------------
int
main (void)
{
    pid_t   pid;
    proc_t *ps;
    int     fd;
    ssize_t size_read;
    char    buffer[sizeof (proc_t)];

    fd = open ("/dev/kmem_ps", O_RDONLY);
    if (fd < 0)
      {
        perror ("ps: cannot open \"/dev/kmem_ps\"");
        exit (0);
      }

    print_proc_head ();
    for (pid = 0; pid < PROCESS_MAX; pid++)
      {
        lseek (fd, (off_t) pid, SEEK_SET);
        size_read = read (fd, buffer, sizeof (proc_t));
        if (size_read < sizeof (proc_t))
          {
            printf ("ps: cannot read \"/dev/kmem_ps\" pid %i", pid);
            perror (NULL);
            continue;
          }
        ps = (proc_t *) buffer;
        if (ps->status > 0)
          {
            ps->name[PATH_MAX-1] = 0; // Terminated string.
            print_proc_pid (ps, pid);
          }
      }

    close (fd);
    return (0);
}
