#include <sys/os16.h>
#include <kernel/devices.h>
#include <sys/types.h>
#include <errno.h>
#include <kernel/memory.h>
#include <kernel/ibm_i86.h>
#include <kernel/proc.h>
#include <string.h>
#include <signal.h>
#include <kernel/k_libc.h>
#include <ctype.h>
#include <kernel/tty.h>
//----------------------------------------------------------------------
ssize_t
dev_kmem (pid_t pid, dev_t device, int rw, off_t offset, void *buffer,
          size_t size, int *eof)
{
  size_t    size_real;
  inode_t  *inode;
  sb_t     *sb;
  file_t   *file;
  void     *start;
  //
  // Only read is allowed.
  //
  if (rw != DEV_READ)
    {
      errset (EIO);                           // I/O error.
      return ((ssize_t) -1);
    }
  //
  // Only positive offset is allowed.
  //
  if (offset < 0)
    {
      errset (EIO);                           // I/O error.
      return ((ssize_t) -1);
    }
  //
  // Read is selected (and is the only access allowed).
  //
  switch (device)
    {
      case DEV_KMEM_PS:
          //
          // Verify if the selected slot can be read.
          //
          if (offset >= PROCESS_MAX)
            {
              errset (EIO);                   // I/O error.
              return ((ssize_t) -1);
            }
          //
          // Correct the size to be read.
          //
          if (sizeof (proc_t) < size)
            {
              size = sizeof (proc_t);
            }
          //  //
          //  // Correct the size to be read.
          //  //
          //  size_real = ((sizeof (proc_t)) * (PROCESS_MAX - offset));
          //  if (size_real < size)
          //    {
          //      size = size_real;
          //    }
          //
          // Get the pointer to the selected slot.
          //
          start = proc_reference ((pid_t) offset);
          break;
      case DEV_KMEM_MMP:
          //
          // Correct the size to be read.
          //
          size_real = (MEM_MAX_BLOCKS/8);
          if (size_real < size)
            {
              size = size_real;
            }
          //
          // Get the pointer to the map.
          //
          start = mb_reference ();
          break;
      case DEV_KMEM_SB:
          //
          // Get a reference to the super block table.
          //
          sb = sb_reference (0);
          //
          // Correct the size to be read.
          //
          if (sizeof (sb_t) < size)
            {
              size = sizeof (sb_t);
            }
          //
          // Get the pointer to the selected super block slot.
          //
          start = &sb[offset];
          break;
      case DEV_KMEM_INODE:
          //
          // Get a reference to the inode table.
          //
          inode = inode_reference (0, 0);
          //
          // Correct the size to be read.
          //
          if (sizeof (inode_t) < size)
            {
              size = sizeof (inode_t);
            }
          //
          // Get the pointer to the selected inode slot.
          //
          start = &inode[offset];
          break;
      case DEV_KMEM_FILE:
          //
          // Get a reference to the file table.
          //
          file = file_reference (0);
          //
          // Correct the size to be read.
          //
          if (sizeof (file_t) < size)
            {
              size = sizeof (file_t);
            }
          //
          // Get the pointer to the selected inode slot.
          //
          start = &file[offset];
          break;
      default:
          errset (ENODEV);            // No such device.
          return ((ssize_t) -1);
    }
  //
  // At this point, data is ready to be copied to the buffer.
  //
  memcpy (buffer, start, size);
  //
  // Return size read.
  //
  return (size);
}
