#ifndef _KERNEL_DIAG_H
#define _KERNEL_DIAG_H 1

#include <stdint.h>
#include <kernel/fs.h>
#include <sys/types.h>
#include <kernel/proc.h>

//----------------------------------------------------------------------
uint8_t  reverse_8_bit  (uint8_t  source);
uint16_t reverse_16_bit (uint16_t source);
uint32_t reverse_32_bit (uint32_t source);

#define reverse_char(s)      ((char)     reverse_8_bit  ((uint8_t)  s))
#define reverse_short(s)     ((short)    reverse_16_bit ((uint16_t) s))
#define reverse_int(s)       ((int)      reverse_16_bit ((uint16_t) s))
#define reverse_long(s)      ((long)     reverse_32_bit ((uint32_t) s))
#define reverse_long_int(s)  ((long int) reverse_32_bit ((uint32_t) s))
//----------------------------------------------------------------------
void print_hex_8  (void *data, size_t elements);
void print_hex_16 (void *data, size_t elements);
void print_hex_32 (void *data, size_t elements);

#define print_hex_char(d, e)      (print_hex_8  (d, e))
#define print_hex_short(d, e)     (print_hex_16 (d, e))
#define print_hex_int(d, e)       (print_hex_16 (d, e))
#define print_hex_long(d, e)      (print_hex_32 (d, e))
#define print_hex_long_int(d, e)  (print_hex_32 (d, e))
//----------------------------------------------------------------------
void print_hex_8_reverse  (void *data, size_t elements);
void print_hex_16_reverse (void *data, size_t elements);
void print_hex_32_reverse (void *data, size_t elements);

#define print_hex_char_reverse(d, e)      (print_hex_8_reverse  (d, e))
#define print_hex_short_reverse(d, e)     (print_hex_16_reverse (d, e))
#define print_hex_int_reverse(d, e)       (print_hex_16_reverse (d, e))
#define print_hex_long_reverse(d, e)      (print_hex_32_reverse (d, e))
#define print_hex_long_int_reverse(d, e)  (print_hex_32_reverse (d, e))
//----------------------------------------------------------------------
void print_segments (void);
void print_kmem     (void);
//----------------------------------------------------------------------
void print_mb_map     (void);
void print_memory_map (void);
//----------------------------------------------------------------------
void print_superblock       (sb_t *sb);
void print_inode            (inode_t *inode);
void print_inode_map        (sb_t *sb, uint16_t *bitmap);
void print_zone_map         (sb_t *sb, uint16_t *bitmap);
void print_inode_head       (void);
void print_inode_list       (void);
void print_inode_zones_head (void);
void print_inode_zones      (inode_t *inode);
void print_inode_zones_list (void);
//----------------------------------------------------------------------
void print_proc_head (void);
void print_proc_pid  (proc_t *ps, pid_t pid);
void print_proc_list (void);
//----------------------------------------------------------------------
void print_file_head (void);
void print_file_num  (int num);
void print_file_list (void);
//----------------------------------------------------------------------
void print_fd_head (void);
void print_fd      (fd_t *fd);
void print_fd_list (pid_t pid);
//----------------------------------------------------------------------
void print_time      (void);
//----------------------------------------------------------------------

#endif
