#include <kernel/proc.h>
#include <kernel/k_libc.h>
#include <errno.h>
//----------------------------------------------------------------------
int
fd_chown (pid_t pid, int fdn, uid_t uid, gid_t gid)
{
    proc_t   *ps;
    inode_t  *inode;
    //
    // Get process.
    //
    ps = proc_reference (pid);
    //
    // Verify if the file descriptor is valid.
    //
    if (ps->fd[fdn].file == NULL)
      {
        errset (EBADF);         // Bad file descriptor.
        return (-1);
      }
    //
    // Reach the inode.
    //
    inode = ps->fd[fdn].file->inode;
    //
    // Verify to be root, as the ability to change group
    // is not taken into consideration.
    //
    if (ps->euid != 0)
      {
        errset (EACCES);        // Permission denied.
        return (-1);
      }
    //
    // Update the ownership.
    //
    if (uid != -1)
      {
        inode->uid     = uid;
        inode->changed = 1;
      }
    if (gid != -1)
      {
        inode->gid     = gid;
        inode->changed = 1;
      }
    //
    // Save the inode.
    //
    inode->changed = 1;
    inode_save (inode);
    //
    // Return.
    //
    return (0);
}
