#include <kernel/proc.h>
#include <kernel/k_libc.h>
#include <errno.h>
#include <fcntl.h>
//----------------------------------------------------------------------
ssize_t
fd_read (pid_t pid, int fdn, void *buffer, size_t count, int *eof)
{
    fd_t  *fd;
    ssize_t  size_read;
    //
    // Get file descriptor.
    //
    fd = fd_reference (pid, &fdn);
    if (fd              == NULL   ||
        fd->file        == NULL   ||
        fd->file->inode == NULL )
      {
        errset (EBADF);                 // Bad file descriptor.
        return ((ssize_t) -1);
      }
    //
    // Check if it is opened for read.
    //
    if (!(fd->file->oflags & O_RDONLY))
      {
        //
        // The file is not opened for read.
        //
        errset (EINVAL);                // Invalid argument.
        return ((ssize_t) -1);
      }
    //
    // It is not a mistake to read a directory, as `dirent.h' is
    // implemented through file descriptors.
    //
    //  //
    //  // Check if it is a directory.
    //  //
    //  if (fd->file->inode->mode & S_IFDIR)
    //    {
    //      errset (EISDIR);                // Is a directory.
    //      return ((ssize_t) -1);
    //    }
    //
    // Check the kind of file to be read and read it.
    //
    if  (S_ISBLK (fd->file->inode->mode)
      || S_ISCHR (fd->file->inode->mode))
      {
        //
        // A device is to be read.
        //
        size_read = dev_io (pid, (dev_t) fd->file->inode->direct[0],
                            DEV_READ, fd->file->offset, buffer, count,
                            eof);
      }
    else if (S_ISREG (fd->file->inode->mode))
      {
        //
        // A regular file is to be read.
        //
        size_read = inode_file_read (fd->file->inode, fd->file->offset,
                                     buffer, count, eof);
      }
    else if (S_ISDIR (fd->file->inode->mode))
      {
        //
        // A directory, is to be read.
        //
        size_read = inode_file_read (fd->file->inode, fd->file->offset,
                                     buffer, count, eof);
      }
    else
      {
        //
        // Unsupported file type.
        //
        errset (E_FILE_TYPE_UNSUPPORTED);       //File type unsupported.
        return ((ssize_t) -1);
      }
    //
    // Update the file descriptor internal offset.
    //
    if (size_read > 0)
      {
        fd->file->offset += size_read;
      }
    //
    // Just return the size read, even if it is an error. Please note
    // that a size of zero might tell that it is the end of file, or
    // just that the read should be retried.
    //
    return (size_read);
}
