#include <kernel/fs.h>
#include <errno.h>
#include <kernel/k_libc.h>
//----------------------------------------------------------------------
blkcnt_t
inode_fzones_read (inode_t *inode, zno_t zone_start,
                   void *buffer, blkcnt_t blkcnt)
{
    unsigned char *destination = (unsigned char *) buffer;
    int            status;        // `zone_read()' return value.
    blkcnt_t       blkcnt_read;   // Zone counter/index.
    zno_t          zone;
    zno_t          fzone;
    //
    // Read the zones into the destination buffer.
    //
    for (blkcnt_read = 0, fzone = zone_start;
         blkcnt_read < blkcnt;
         blkcnt_read++, fzone++)
      {
        //
        // Calculate the zone number, from the file-zone, reading the
        // inode. If a zone is not really allocated, the result is zero
        // and is valid.
        //
        zone = inode_zone (inode, fzone, 0);
        if (zone == ((zno_t) -1))
          {
            //
            // This is an error. Return the read zones quantity.
            //
            return (blkcnt_read);
          }
        //
        // Update the destination buffer pointer.
        //
        destination += (blkcnt_read * inode->sb->blksize);
        //
        // Read the zone inside the destination buffer, but if the zone
        // is zero, a zeroed zone must be filled.
        //
        if (zone == 0)
          {
            memset (destination, 0, (size_t) inode->sb->blksize);
          }
        else
          {
            status = zone_read (inode->sb, zone, destination);
            if (status != 0)
              {
                //
                // Could not read the requested zone: return the zones
                // read correctly.
                //
                errset (EIO);               // I/O error.
                return (blkcnt_read);
              }
          }
      }
    //
    // All zones read correctly inside the buffer.
    //
    return (blkcnt_read);
}
