.global __int13_03
;-----------------------------------------------------------------------
.text
;-----------------------------------------------------------------------
; INT 0x13 - disk - write sectors to disk
;       AH = 0x03
;       AL = number of sectors to write (must be nonzero)
;       CH = cylinder number (0-255)
;       CL bit 6-7 =
;            cylinder number (256-1023)
;       CL bit 0-5 =
;            sector number   (1-63)
;       DH = head number     (0-255)
;       DL = drive number (bit 7 set for hard disk)
;       ES:BX -> data buffer
; Return:
;       CF set on error
;       CF clear if successful
;       AH = status (0x00 if successful)
;       AL = number of sectors transferred (only valid if CF set for
;            some BIOSes)
;-----------------------------------------------------------------------
.align 2
__int13_03:
    enter #2, #0        ; 1 local variable.
    pushf
    cli
    pusha
    mov   ax, ds        ; Set ES the same as DS.
    mov   es, ax        ;
    mov   ax,  8[bp]    ; 3rd arg (cylinder). It must be splitted and
    mov   ch, al        ; assigned to the right registers.
    mov   cl, ah        ; 
    shl   cl, 1         ;
    shl   cl, 1         ;
    shl   cl, 1         ;
    shl   cl, 1         ;
    shl   cl, 1         ;
    shl   cl, 1         ;
    add   cl, 12[bp]    ; 5th arg (sector).
    mov   dl,  4[bp]    ; 1st arg (drive).
    mov   al,  6[bp]    ; 2nd arg (sectors to be written).
    mov   dh, 10[bp]    ; 4th arg (head).
    mov   bx, 14[bp]    ; 6th arg (buffer pointer).
    mov   ah, #0x03
    int   #0x13
    mov   -2[bp], ax    ; save AX.
    popa
    popf
    mov   ax, -2[bp]    ; AX is the function return value.
    leave
    ret
