#include <kernel/main.h>
#include <errno.h>
#include <fcntl.h>
#include <kernel/main/build.h>
#include <kernel/diag.h>
#include <kernel/fs.h>
#include <kernel/ibm_i86.h>
#include <kernel/k_libc.h>
#include <kernel/proc.h>
#include <libgen.h>
#include <stdlib.h>
#include <sys/os16.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
    unsigned int key;
    pid_t        pid;
    char        *exec_argv[2];
    int          status;
    int          exit;
    //
    // Reset video and select the initial console.
    //
    tty_init ();
    //
    // Show compilation date and time.
    //
    k_printf ("os16 build %s ram %i Kibyte\n", BUILD_DATE, int12 ());
    //
    // Set up disk management.
    //
    dsk_setup ();
    //
    // Clear heap for diagnosis.
    //
    heap_clear ();
    //
    // Set up process management. Process set up need the file system
    // root directory already available.
    //
    proc_init ();
    //
    // The kernel will run interactively.
    //
    menu ();
    //
    for (exit = 0; exit == 0;)
      {
        //
        // While in kernel code, timer interrupt don't start the
        // scheduler. The kernel must leave control to the scheduler
        // via a null system call.
        //
        sys (SYS_0, NULL, 0);
        //
        // Back to work: read the keyboard from the TTY device.
        //
        dev_io ((pid_t) 0, DEV_TTY, DEV_READ, 0L, &key, 1, NULL);
        //
        // Depending on the key, do something.
        //
        if (key == 0)
          {
            //
            // No key is ready in the buffer keyboard.
            //
            continue;
          }
        else
          {
            //
            // Move back the cursor, so that next print will overwrite
            // it.
            //
            k_printf ("\b");
          }
        //
        // A key was pressed: start to check what it was.
        //
        switch (key)
          {
            case 'h':
                menu ();
                break;
            case '1':
                k_kill ((pid_t) 1, SIGKILL);                   // init
                break;
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
            case '8':
            case '9':
                k_kill ((pid_t) (key - '0'),  SIGTERM);        // others
                break;
            case 'A':
            case 'B':
            case 'C':
            case 'D':
            case 'E':
            case 'F':
                k_kill ((pid_t) (key - 'A' + 10),  SIGTERM);   // others
                break;
            case 'a':
                run ("/bin/aaa", NULL, NULL);
                break;
            case 'b':
                run ("/bin/bbb", NULL, NULL);
                break;
            case 'c':
                run ("/bin/ccc", NULL, NULL);
                break;
            case 'f':
                print_file_list ();
                break;
            case 'm':
                status = path_mount ((uid_t) 0, "/dev/dsk1", "/usr",
                                     MOUNT_DEFAULT);
                if (status < 0)
                  {
                    k_perror (NULL);
                  }
                break;
            case 'M':
                status = path_umount ((uid_t) 0, "/usr");
                if (status < 0)
                  {
                    k_perror (NULL);
                  }
                break;
            case 'n':
                print_inode_list ();
                break;
            case 'N':
                print_inode_zones_list ();
                break;
            case 'l':
                k_kill ((pid_t) 1, SIGCHLD);
                break;
            case 'p':
                k_printf ("\n");
                print_proc_list ();
                print_segments ();
                k_printf (" ");
                print_kmem ();
                k_printf (" ");
                print_time ();
                k_printf ("\n");
                print_mb_map ();
                k_printf ("\n");
                break;
            case 'x':
                exit = 1;
                break;
            case 'q':
                k_printf ("System halted!\n");
                return (0);
                break;
          }
      }
    //
    // Load init.
    //
    exec_argv[0] = "/bin/init";
    exec_argv[1] = NULL;
    pid = run ("/bin/init", exec_argv, NULL);
    //
    // Just sleep.
    //
    while (1)
      {
        sys (SYS_0, NULL, 0);
      }
    //
    k_printf ("System halted!\n");
    return (0);
}
