#include <unistd.h>
#include <sys/types.h>
#include <sys/os16.h>
#include <errno.h>
#include <stddef.h>
#include <string.h>
//----------------------------------------------------------------------
char *
getcwd (char *buffer, size_t size)
{
    sysmsg_uarea_t msg;
    //
    // Check arguments: the buffer must be given.
    //
    if (buffer == NULL)
      {
        errset (EINVAL);
        return (NULL);
      }
    //
    // Make shure that the last character, inside the working directory
    // path is a null character.
    //
    msg.path_cwd[PATH_MAX-1] = 0;
    //
    // Just get the user area data.
    //
    sys (SYS_UAREA, &msg, (sizeof msg));
    //
    // Check that the path is still correctly terminated. If it isn't,
    // the path is longer than the implementation limits, and it is
    // really *bad*.
    //
    if (msg.path_cwd[PATH_MAX-1] != 0)
      {
        errset (E_LIMIT);	// Exceeded implementation limits.
	return (NULL);
      }
    //
    // If the path is larger than the buffer size, return an error.
    // Please note that the parameter `size' must include the
    // terminating null character, so, if the string is equal to
    // the size, it is already beyond the size limit.
    //
    if (strlen (msg.path_cwd) >= size)
      {
	errset (ERANGE);	// Result too large.
	return (NULL);
      }
    //
    // Everything is fine, so, copy the path to the buffer and return.
    //
    strncpy (buffer, msg.path_cwd, size);
    return (buffer);
}
